# Installing Required Software on your Development System

Unlike the tutorials, where all required software is encapsulated in Docker containers, software development also requires other software installed directly on your *Development System*.  This follows the terminology from the [OS installation](install_os.md) page -- the development system is the Ubuntu LTS installation that runs either directly on your computer or in a virtual machine.

The software installation process is mostly automated through [Ansible Playbooks](https://docs.ansible.com/ansible/latest/index.html) ("a repeatable, re-usable, simple configuration management and multi-machine deployment system, one that is well suited to deploying complex applications").

## Setting up SSH

You will be running Ansible Playbooks on your Development System, to install software on the same Development System.  However, Ansible is designed to be able to install software on remote hosts as well, so the playbooks will use SSH even for a local installation.

Open a terminal window on your Development System.  Execute the following command to update the list of packages available for your system and to install the OpenSSH server:
```bash
sudo apt update
sudo apt install openssh-server
```

Test if you can SSH from the development system into itself without providing the password. This is necessary for the remaining steps described below.  
```bash
ssh ${USER}@localhost
```
If you are asked to enter a password, cancel the command (Ctrl+C) and execute the following command to install the needed SSH key:
```bash
ssh-copy-id -i ${USER}@localhost
```

If you don't have an SSH key generated on the development system yet, execute:
```bash
ssh-keygen
```

Then repeat the previous step. 

You should now be able to log in from the development system to itself using `ssh ${USER}@localhost` without a password.  (This does not mean that you can log in from elsewhere without a password -- you first need to log in to the system with a password, and *then* you have access to the SSH key that is needed to log in from your account on the development system to the *same* account on the development system.)

## Installing Ansible

If you use Ubuntu 18.04, you need to do the following in order to get access to a repository containing the Ansible software:

```bash
## sudo install --reinstall ca-certificates # Had to do this for the apt-add-repository to work
# Only for Ubuntu 18.04!
sudo apt install software-properties-common
sudo apt-add-repository --yes --update ppa:ansible/ansible
```

Regardless of Ubuntu version, continue as follows:

```bash
sudo apt install ansible
```

## Installing software using Ansible Playbooks

Clone the `lrs_playbooks` repo in some suitable location to get access to the actual playbooks for WARA PS:

```bash
sudo apt install git
cd ...some suitable location...
git clone https://gitlab.liu.se/lrs/lrs_playbooks.git
```

Run the `wara_develop` playbook as follows to install and update the WARA PS developer software. This can take several minutes to complete.

- Make sure you replace the last argument with the actual password of the root/administrator on the development system.

+++++++++++ The root/admin or an ordinary user account doing sudo?  Do you need to configure sudo in some specific way so that the user has access to sudo? +++++++++++++++

```bash
cd lrs_playbooks
git pull
ansible-playbook -i hosts playbooks/wara_develop.yml -e user=${USER} -e install_user=${USER} -e ansible_sudo_pass=******
```

Among other things, this will:

- Install certain software such as cmake, g++, gcc, git, screen, joe if not already installed 
- Install the [DJI SDK](https://github.com/dji-sdk/Onboard-SDK.git)
- Install [Docker](https://www.docker.com/)
- Install [ROS 1](https://www.ros.org/)
- Clone a number of development repositories for the ROS-based LRS system 
- Compile some source files


## Finishing Up

Add the following commands at the end of the ```~/.bashrc``` file on your development system:
```bash
export WORLD_ORIGIN_LAT=57.7605573519 
export WORLD_ORIGIN_LON=16.6827607783
export WORLD_ORIGIN_ELEVATION=29.8

#source /opt/ros/melodic/setup.bash
source ${HOME}/wara_ws/devel/setup.bash
export PATH=${HOME}/lrs_devenv_common/script:${PATH}
```

Then log in and log out to the development system for the docker commands to work.  Test this with:

```bash
docker ps
```
It should show:
```bash
tompe@philo:~$ docker ps
CONTAINER ID        IMAGE               COMMAND             CREATED             STATUS              PORTS               NAMES
tompe@philo:~$ 
```

At this point your system has the necessary software installed.

You can use the following commands on the development system to test the 3D graphics hardware and driver setup on your system:

```bash
glxgears
glmark2
```

If the above commands execute without errors you can test that Gazebo
works correctly on your development system:

```bash
roslaunch lrs_wara_gazebo gazebo.launch
roslaunch lrs_wara_gazebo dji_to_gazebo.launch name:=dji0 with_laser:=true x:=5
```

All files to run these tests should be available in the `lrs_wara_gazebo`
repository 
which is installed in `wara_ws` by the Playbook above. The first command
starts Gazebo and the second command adds a model of a DJI M100 to the
Gazebo world.
